#!/usr/bin/env python
# vim: set ff=unix expandtab: shiftwidth=4 fileencoding=utf-8
"""
Intelligent RObotics 2014 - source code skeleton
(c) 2014 Czech Technical University, Faculty of Electrical Engineering, Department of Cybernetics

TASK 1
======

This is the sample source code for task 1. Students are free to change it in order to solve the task.

Other use is allowed under the terms of the New BSD license.
"""
__author__ =  'Martin Pecka <peckama2 fel.cvut.cz>'
__version__=  '2014.1.1'
__license__ = 'BSD'


# Ros libraries
import roslib; roslib.load_manifest('task1')
import rospy;

# Python libs
import sys, os

# numpy and scipy
import numpy as np
from scipy.io import savemat

# OpenCV
import cv2

# Ros Messages
from sensor_msgs.msg import CompressedImage   # this is for the rgb
from sensor_msgs.msg import Image             # this is for depth/thermo
from sensor_msgs.msg import CameraInfo        # Kinect info

class image_recorder:

    def __init__(self):
        '''Initialize ROS subscribers'''
        
        # IMPLEMENT
        # you have to fill the right topic names into the subscribers
        # rosbag play + rostopic list, rosmsg type and so on may be of some help
        
        # the depth image on the Asus Xtion Pro device (aka Kinect)
        self.subscriber_depth = rospy.Subscriber("put_topic_name_here",
            Image, self.callback_depth,  queue_size = 1)
        
        # the RGB camera on the Asus Xtion Pro device (aka Kinect)
        self.subscriber_rgb = rospy.Subscriber("put_topic_name_here",
            CompressedImage, self.callback_rgb,  queue_size = 1)

        # metadata about the Asus Xtion Pro device (aka Kinect) - for reading the calibration matrix
        self.subscriber_camera_info = rospy.Subscriber("put_topic_name_here",
            CameraInfo, self.callback_camera_info, queue_size = 1)
        
        # the infrared camera Micro-Epsilon thermoIMAGER TIM 160
        self.subscriber_thermo = rospy.Subscriber("put_topic_name_here",
            Image, self.callback_thermo,  queue_size = 1)
        
        # member variables

        self.depth_images = []
        self.rgb_images = []
        self.thermo_images = []
        self.kinect_K = None

    def callback_depth(self, ros_data):
        '''Callback function that records depth information.''' 
        
        #### the z-values are saved as 16bit ints and represent depth in milimeters
        linearized_data = np.fromstring(ros_data.data, np.uint16)

        #reshaping 1D array to 2D
        self.depth_images.append(np.reshape(linearized_data,(ros_data.height,ros_data.width)))

        print 'Received depth image, now having', len(self.depth_images), 'of them'
        
    def callback_thermo(self, ros_data):
        '''Callback function that records temperature data.''' 
        
        #### the temperature is stored as 32bit floats and the numbers represent the temperature in °C
        linearized_data = np.fromstring(ros_data.data, np.float32)

        # change shape 
        self.thermo_images.append(np.reshape(linearized_data,(ros_data.height,ros_data.width)))

        print 'Received thermo image, now having', len(self.thermo_images), 'of them'
    
    def callback_rgb(self, ros_data):
        '''Callback function that records the RGB image.''' 
    
        # the image is transmitted in JPEG and so we need to read it as bytes first
        raw_bytes = np.fromstring(ros_data.data, np.uint8)

        # let OpenCV decode the JPEG and create Y*X*3 array of pixels
        self.rgb_images.append(cv2.imdecode(raw_bytes, cv2.CV_LOAD_IMAGE_COLOR))

        print 'Received RGB image, now having', len(self.rgb_images), 'of them'

    def callback_camera_info(self, ros_data):
        '''Callback for recording the Asus Xtion PRO camera calibration matrix'''

        if (self.kinect_K is None):
            # IMPLEMENT
            self.kinect_K = # fill in the right expression

    def main_loop(self, num_frames):

        print 'Ready, please play the bag file, recording', num_frames, 'frames'

        # run the main ROS loop until we load all the images

        clean_shutdown = False # if Ctrl-C is pushed before all the images are recorded, this remains False

        while not rospy.is_shutdown():           
            if(
                    len(self.depth_images) == num_frames and 
                    len(self.rgb_images) == num_frames and 
                    len(self.thermo_images) == num_frames and 
                    self.kinect_K is not None):

                clean_shutdown = True
                break

        if not clean_shutdown:
            print 'Shutdown forced, exiting.'
            return

        print 'Image capture done, saving.'

        # now all the images are loaded, so save them to .mat files
        base_path = os.path.dirname(os.path.realpath(__file__))
        savemat(base_path + '/../data/thermo_images.mat', {'thermo_images': self.thermo_images}, do_compression=True)
        savemat(base_path + '/../data/rgb_images.mat', {'rgb_images': self.rgb_images}, do_compression=True)
        savemat(base_path + '/../data/depth_images.mat', {'depth_images': self.depth_images}, do_compression=True)
        savemat(base_path + '/../data/kinect_K.mat', {'kinect_K': self.kinect_K}, do_compression=True)
        
        print 'Images were successfully saved.'

def main(args):
    # initialize and run ROS node
    recorder = image_recorder()
    rospy.init_node('image_recorder', anonymous=True)

    # load the ~num_frames parameter, which specifies how much frames from the bag file should be recorded
    if not rospy.has_param('~num_frames'):
        print 'You forgot to specify the _num_frames:=number parameter'
        return 1
    num_frames = rospy.get_param('~num_frames')

    # run the main loop
    recorder.main_loop(num_frames)

if __name__ == '__main__':
    main(sys.argv)

